-- Copyright 2016 Stanford University
--
-- Licensed under the Apache License, Version 2.0 (the "License");
-- you may not use this file except in compliance with the License.
-- You may obtain a copy of the License at
--
--     http://www.apache.org/licenses/LICENSE-2.0
--
-- Unless required by applicable law or agreed to in writing, software
-- distributed under the License is distributed on an "AS IS" BASIS,
-- WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-- See the License for the specific language governing permissions and
-- limitations under the License.

import "regent"

local c = regentlib.c
local std = terralib.includec("stdlib.h")

-- Terra is another language embedded in Lua, just like Regent.  Regent and Terra have similar syntax (by design),
-- but Terra is a much smaller language, without features for parallelism.  In fact, Terra is designed to be isomorphic
-- to C, and it also includes extensive support for vector operations (not illustrated here).  Thus, Terra is much better
-- than Regent for leaf tasks, tasks that are themselves designed to run on one core (and don't call any other tasks).  Leaf
-- tasks that need to be fast should be written in Terra; in general, Terra's more restricted language means that standard
-- C compilation techniques will produce significantly better code than the equivalent Regent program.
terra terra_hits(iterations: int64)
     var total: int64 = 0
     for i = 1, iterations do
          var x: double = std.drand48()
     	  var y: double = std.drand48()
          if (x * x) + (y * y) <= 1.0 then
     	     total = total + 1
          end
     end
     return total
end

task hits(iterations: int64)
     return terra_hits(iterations)
end

task main()

     var iterations: int64 = 2000
     
     var hits1 = hits(iterations)
     var hits2 = hits(iterations)
     var hits3 = hits(iterations)
     var hits4 = hits(iterations)
     var totalhits = hits1 + hits2 + hits3 + hits4
     c.printf("The area of a unit circle is approximately: %5.4f\n", totalhits / float(iterations))
end

regentlib.start(main)