-- Copyright 2016 Stanford University
--
-- Licensed under the Apache License, Version 2.0 (the "License");
-- you may not use this file except in compliance with the License.
-- You may obtain a copy of the License at
--
--     http://www.apache.org/licenses/LICENSE-2.0
--
-- Unless required by applicable law or agreed to in writing, software
-- distributed under the License is distributed on an "AS IS" BASIS,
-- WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-- See the License for the specific language governing permissions and
-- limitations under the License.

import "regent"

local c = regentlib.c

fspace Node {
 id: int64
}

fspace Edge(r: region(Node)) {
    source_node : ptr(Node, r),
    dest_node: ptr(Node, r)
}

task main()
   var Num_Parts = 4
   var Num_Elements = 20

   var nodes = region(ispace(ptr, Num_Elements), Node)
   var edges = region(ispace(ptr, Num_Elements-1), Edge(nodes))

   var i = 0
   for node in nodes do
        node.id = i
        i += 1
   end

   var j = 0
   for edge in edges do
       for n in nodes do
           for m in nodes do
	          if n.id == j and m.id == n.id + 1 then
                     edge.source_node = n
                     edge.dest_node = m
		  end
            end
	end
	j += 1
    end

   var colors = ispace(int1d, Num_Parts)
   var edge_partition = partition(equal, edges, colors)

   for color in edge_partition.colors do
     c.printf("Edge subregion %d: ", color)
     for e in edge_partition[color] do
        c.printf("(%d,%d) ", e.source_node.id, e.dest_node.id)
     end
     c.printf("\n")
   end

   var node_partition1 = image(nodes, edge_partition, edges.dest_node)
   var node_partition2 = image(nodes, edge_partition, edges.source_node)
--
--  A variation on example24, now keeping those source nodes of edges that
--  are not also destination nodes.
--
   var node_partition = node_partition2 - node_partition1

   for color in node_partition.colors do
     c.printf("Node subregion %d: ", color)
     for n in node_partition[color] do
        c.printf("%d ", n.id)
     end
     c.printf("\n")
   end




end
  
regentlib.start(main)