-- Copyright 2016 Stanford University
--
-- Licensed under the Apache License, Version 2.0 (the "License");
-- you may not use this file except in compliance with the License.
-- You may obtain a copy of the License at
--
--     http://www.apache.org/licenses/LICENSE-2.0
--
-- Unless required by applicable law or agreed to in writing, software
-- distributed under the License is distributed on an "AS IS" BASIS,
-- WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-- See the License for the specific language governing permissions and
-- limitations under the License.

-- Inspired by https://github.com/ParRes/Kernels/tree/master/LEGION/Stencil

import "regent"

local c = regentlib.c

function gen_expr(c,x)
  return `c * x
end



function create_expr(num, v)
  local value
  for i = 1,num do
    if value then
      value = `value + v
    else
      value = `v
    end
  end
  return value
end

terra scale(a: float): float
  -- This structure will lead to an error.  Because create_expr(...) is inside a splice [...],
  -- its call is evaluated when scale is defined.  But ITERATE is a terra variable so does not have a value
  -- until scale is called at runt time.  Thus, ITERATE is passed to create_expr as a piece of code (i.e., a variable
  -- named ITERATE) and create_expr throws an error because it needs the value, not the name, of its first argument.
  var ITERATE = 6
  return [create_expr(ITERATE,a)]
end

scale:printpretty()
c.printf("Answer: %f\n", scale(7.0))

