-- Copyright 2016 Stanford University
--
-- Licensed under the Apache License, Version 2.0 (the "License");
-- you may not use this file except in compliance with the License.
-- You may obtain a copy of the License at
--
--     http://www.apache.org/licenses/LICENSE-2.0
--
-- Unless required by applicable law or agreed to in writing, software
-- distributed under the License is distributed on an "AS IS" BASIS,
-- WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-- See the License for the specific language governing permissions and
-- limitations under the License.

-- Inspired by https://github.com/ParRes/Kernels/tree/master/LEGION/Stencil

import "regent"

local c = regentlib.c

local function create_stencil(width, s, j)
  local value
  for i =  -width,width do
    local component = rexpr s[ i + j ] end
    if value then
      value = rexpr value + component end
    else
      value = rexpr component end
    end
   end
   return value
end

STENCIL_WIDTH = 10
local function make_task(radius)
  local task toplevel()
    var vsize = 100000
    var nt = 1000
    var vector = ispace(int1d, vsize)
    var source = region(vector, float)
    var dest = region(vector,float)
    fill(source, 1.0)
    fill(dest,0.0)
    
    for i = 1,nt do
      for j = STENCIL_WIDTH, vsize - STENCIL_WIDTH do
	dest[j] = [ create_stencil(STENCIL_WIDTH, source, j) ]
      end
    end
for i = 0, 20 do
    c.printf("%d: %f\n", 0, dest[i])
end
  end
  return toplevel
end

local main = make_task(10)
main:printpretty()
regentlib.start(main)
